package kr.co.vividnext.sodalive.admin.content

import kr.co.vividnext.sodalive.admin.content.curation.AdminContentCurationRepository
import kr.co.vividnext.sodalive.aws.cloudfront.AudioContentCloudFront
import kr.co.vividnext.sodalive.common.SodaException
import org.springframework.beans.factory.annotation.Value
import org.springframework.data.domain.Pageable
import org.springframework.data.repository.findByIdOrNull
import org.springframework.stereotype.Service
import org.springframework.transaction.annotation.Transactional

@Service
class AdminContentService(
    private val repository: AdminContentRepository,
    private val audioContentCloudFront: AudioContentCloudFront,
    private val curationRepository: AdminContentCurationRepository,

    @Value("\${cloud.aws.cloud-front.host}")
    private val coverImageHost: String
) {
    fun getAudioContentList(pageable: Pageable): GetAdminContentListResponse {
        val totalCount = repository.getAudioContentTotalCount()
        val audioContentAndThemeList = repository.getAudioContentList(
            offset = pageable.offset,
            limit = pageable.pageSize.toLong()
        )

        val audioContentList = audioContentAndThemeList
            .asSequence()
            .map {
                val tags = repository
                    .getHashTagList(audioContentId = it.audioContentId)
                    .joinToString(" ") { tag -> tag }
                it.tags = tags
                it
            }
            .map {
                it.contentUrl = audioContentCloudFront.generateSignedURL(
                    resourcePath = it.contentUrl,
                    expirationTime = 1000 * 60 * 60 * (it.remainingTime.split(":")[0].toLong() + 2)
                )
                it
            }
            .map {
                it.coverImageUrl = "$coverImageHost/${it.coverImageUrl}"
                it
            }
            .toList()

        return GetAdminContentListResponse(totalCount, audioContentList)
    }

    fun searchAudioContent(searchWord: String, pageable: Pageable): GetAdminContentListResponse {
        if (searchWord.length < 2) throw SodaException("2글자 이상 입력하세요.")
        val totalCount = repository.getAudioContentTotalCount(searchWord)
        val audioContentAndThemeList = repository.getAudioContentList(
            offset = pageable.offset,
            limit = pageable.pageSize.toLong(),
            searchWord = searchWord
        )

        val audioContentList = audioContentAndThemeList
            .asSequence()
            .map {
                val tags = repository
                    .getHashTagList(audioContentId = it.audioContentId)
                    .joinToString(" ") { tag -> tag }
                it.tags = tags
                it
            }
            .map {
                it.contentUrl = audioContentCloudFront.generateSignedURL(
                    resourcePath = it.contentUrl,
                    expirationTime = 1000 * 60 * 60 * (it.remainingTime.split(":")[0].toLong() + 2)
                )
                it
            }
            .map {
                it.coverImageUrl = "$coverImageHost/${it.coverImageUrl}"
                it
            }
            .toList()

        return GetAdminContentListResponse(totalCount, audioContentList)
    }

    @Transactional
    fun updateAudioContent(request: UpdateAdminContentRequest) {
        val audioContent = repository.findByIdOrNull(id = request.id)
            ?: throw SodaException("없는 콘텐츠 입니다.")

        if (request.isDefaultCoverImage) {
            audioContent.coverImage = "profile/default_profile.png"
        }

        if (request.isActive != null) {
            audioContent.isActive = request.isActive
        }

        if (request.isAdult != null) {
            audioContent.isAdult = request.isAdult
        }

        if (request.isCommentAvailable != null) {
            audioContent.isCommentAvailable = request.isCommentAvailable
        }

        if (request.title != null) {
            audioContent.title = request.title
        }

        if (request.detail != null) {
            audioContent.detail = request.detail
        }

        if (request.curationId != null) {
            val curation = curationRepository.findByIdAndActive(id = request.curationId)
            audioContent.curation = curation
        }
    }
}
