package kr.co.vividnext.sodalive.admin.explorer

import kr.co.vividnext.sodalive.common.SodaException
import kr.co.vividnext.sodalive.explorer.section.ExplorerSection
import kr.co.vividnext.sodalive.explorer.section.ExplorerSectionCreatorTag
import kr.co.vividnext.sodalive.member.tag.MemberTagRepository
import org.springframework.data.domain.Pageable
import org.springframework.data.repository.findByIdOrNull
import org.springframework.stereotype.Service
import org.springframework.transaction.annotation.Transactional

@Service
@Transactional(readOnly = true)
class AdminExplorerService(
    private val repository: AdminExplorerSectionRepository,
    private val memberTagRepository: MemberTagRepository
) {
    @Transactional
    fun createExplorerSection(request: CreateExplorerSectionRequest): Long {
        if (request.title.isBlank()) throw SodaException("제목을 입력하세요.")

        val findExplorerSection = repository.findByTitle(request.title)
        if (findExplorerSection != null) throw SodaException("동일한 제목이 있습니다.")

        val explorerSection = ExplorerSection(title = request.title, isAdult = request.isAdult)
        explorerSection.coloredTitle = request.coloredTitle
        explorerSection.color = request.color

        val tags = mutableListOf<ExplorerSectionCreatorTag>()
        request.tagList.forEach {
            val findTag = memberTagRepository.findByTag(it)
            if (findTag != null) {
                val tag = ExplorerSectionCreatorTag()
                tag.explorerSection = explorerSection
                tag.tag = findTag
                tags.add(tag)
            }
        }

        if (tags.size <= 0) throw SodaException("관심사를 선택하세요.")
        explorerSection.tags = tags

        return repository.save(explorerSection).id!!
    }

    @Transactional
    fun updateExplorerSection(request: UpdateExplorerSectionRequest) {
        if (
            request.title == null &&
            request.isAdult == null &&
            request.tagList == null &&
            request.color == null &&
            request.coloredTitle == null &&
            request.isActive == null
        ) {
            throw SodaException("변경사항이 없습니다.")
        }

        val explorerSection = repository.findByIdOrNull(request.id)
            ?: throw SodaException("해당하는 섹션이 없습니다.")

        if (request.title != null) {
            if (request.title.isBlank()) throw SodaException("올바른 제목을 입력하세요.")
            explorerSection.title = request.title
        }

        if (request.isActive != null) {
            explorerSection.isActive = request.isActive
        }

        if (request.isAdult != null) {
            explorerSection.isAdult = request.isAdult
        }

        if (request.color != null) {
            explorerSection.color = request.color
        }

        if (request.coloredTitle != null) {
            explorerSection.coloredTitle = request.coloredTitle
        }

        if (request.tagList != null) {
            val requestTagList = request.tagList.toMutableList()
            val tags = explorerSection.tags.filter {
                requestTagList.contains(it.tag!!.tag)
                requestTagList.remove(it.tag!!.tag)
            }.toMutableList()

            requestTagList.forEach {
                val findTag = memberTagRepository.findByTag(it)
                if (findTag != null) {
                    val tag = ExplorerSectionCreatorTag()
                    tag.explorerSection = explorerSection
                    tag.tag = findTag
                    tags.add(tag)
                }
            }

            if (tags.size <= 0) throw SodaException("관심사를 입력하세요.")
            if (tags != explorerSection.tags) {
                explorerSection.tags.clear()
                explorerSection.tags.addAll(tags)
            }
        }
    }

    @Transactional
    fun updateExplorerSectionOrders(firstOrders: Int, ids: List<Long>) {
        for (index in ids.indices) {
            val explorerSection = repository.findByIdOrNull(ids[index])

            if (explorerSection != null) {
                explorerSection.orders = firstOrders + index
            }
        }
    }

    fun getExplorerSections(pageable: Pageable): GetAdminExplorerSectionResponse {
        val totalCount = repository.totalCount()
        val explorerSectionItemList = repository
            .findAllWithPaging(pageable.offset, pageable.pageSize.toLong())
            .map {
                GetAdminExplorerSectionResponseItem(
                    id = it.id!!,
                    title = it.title,
                    coloredTitle = it.coloredTitle ?: "",
                    color = it.color ?: "",
                    isAdult = it.isAdult,
                    isActive = it.isActive,
                    tags = it.tags
                        .asSequence()
                        .filter { explorerSectionTag -> explorerSectionTag.tag!!.isActive }
                        .map { explorerSectionTag -> explorerSectionTag.tag!!.tag }
                        .toList()
                )
            }

        return GetAdminExplorerSectionResponse(
            totalCount = totalCount,
            explorerSectionItemList = explorerSectionItemList
        )
    }
}
