package kr.co.vividnext.sodalive.admin.live

import com.amazonaws.services.s3.model.ObjectMetadata
import kr.co.vividnext.sodalive.aws.s3.S3Uploader
import kr.co.vividnext.sodalive.can.CanRepository
import kr.co.vividnext.sodalive.can.charge.Charge
import kr.co.vividnext.sodalive.can.charge.ChargeRepository
import kr.co.vividnext.sodalive.can.charge.ChargeStatus
import kr.co.vividnext.sodalive.can.payment.Payment
import kr.co.vividnext.sodalive.can.payment.PaymentGateway
import kr.co.vividnext.sodalive.can.payment.PaymentStatus
import kr.co.vividnext.sodalive.can.use.CanUsage
import kr.co.vividnext.sodalive.can.use.UseCanCalculateRepository
import kr.co.vividnext.sodalive.can.use.UseCanCalculateStatus
import kr.co.vividnext.sodalive.common.SodaException
import kr.co.vividnext.sodalive.fcm.FcmEvent
import kr.co.vividnext.sodalive.fcm.FcmEventType
import kr.co.vividnext.sodalive.live.recommend.RecommendLiveCreatorBanner
import kr.co.vividnext.sodalive.live.recommend.RecommendLiveCreatorBannerRepository
import kr.co.vividnext.sodalive.live.reservation.LiveReservationRepository
import kr.co.vividnext.sodalive.live.room.cancel.LiveRoomCancel
import kr.co.vividnext.sodalive.live.room.cancel.LiveRoomCancelRepository
import kr.co.vividnext.sodalive.member.MemberRepository
import kr.co.vividnext.sodalive.utils.generateFileName
import org.springframework.beans.factory.annotation.Value
import org.springframework.context.ApplicationEventPublisher
import org.springframework.data.domain.Pageable
import org.springframework.data.repository.findByIdOrNull
import org.springframework.stereotype.Service
import org.springframework.transaction.annotation.Transactional
import org.springframework.web.multipart.MultipartFile
import java.time.LocalDateTime
import java.time.ZoneId
import java.time.format.DateTimeFormatter

@Service
class AdminLiveService(
    private val recommendCreatorBannerRepository: RecommendLiveCreatorBannerRepository,
    private val roomCancelRepository: LiveRoomCancelRepository,
    private val repository: AdminLiveRoomQueryRepository,
    private val memberRepository: MemberRepository,
    private val s3Uploader: S3Uploader,

    private val useCanCalculateRepository: UseCanCalculateRepository,
    private val reservationRepository: LiveReservationRepository,
    private val chargeRepository: ChargeRepository,
    private val canRepository: CanRepository,

    private val applicationEventPublisher: ApplicationEventPublisher,

    @Value("\${cloud.aws.s3.bucket}")
    private val bucket: String,
    @Value("\${cloud.aws.cloud-front.host}")
    private val coverImageHost: String
) {
    fun getLiveList(): GetLiveResponse {
        return GetLiveResponse(
            liveList = repository.getLiveRoomList()
                .asSequence()
                .map {
                    GetLiveResponseItem(
                        id = it.id!!,
                        title = it.title,
                        content = it.notice,
                        managerNickname = it.member!!.nickname,
                        coverImageUrl = if (it.coverImage!!.startsWith("https://")) {
                            it.coverImage!!
                        } else {
                            "$coverImageHost/${it.coverImage!!}"
                        },
                        channelName = it.channelName ?: "",
                        type = it.type,
                        password = it.password,
                        isAdult = it.isAdult
                    )
                }
                .toList()
        )
    }

    fun getRecommendCreator(pageable: Pageable): GetAdminRecommendCreatorResponse {
        val dateTimeFormatter = DateTimeFormatter.ofPattern("yyyy-MM-dd HH:mm")

        val totalCount = repository.getRecommendCreatorTotalCount()

        val recommendCreatorList = repository
            .getRecommendCreatorList(pageable)
            .asSequence()
            .map {
                GetAdminRecommendCreatorResponseItem(
                    it.id!!,
                    "$coverImageHost/${it.image}",
                    it.creator!!.id!!,
                    it.creator!!.nickname,
                    it.startDate
                        .atZone(ZoneId.of("UTC"))
                        .withZoneSameInstant(ZoneId.of("Asia/Seoul"))
                        .toLocalDateTime()
                        .format(dateTimeFormatter),
                    it.endDate
                        .atZone(ZoneId.of("UTC"))
                        .withZoneSameInstant(ZoneId.of("Asia/Seoul"))
                        .toLocalDateTime()
                        .format(dateTimeFormatter),
                    it.isAdult
                )
            }
            .toList()

        return GetAdminRecommendCreatorResponse(
            totalCount = totalCount,
            recommendCreatorList = recommendCreatorList
        )
    }

    @Transactional
    fun createRecommendCreatorBanner(
        image: MultipartFile,
        creatorId: Long,
        startDateString: String,
        endDateString: String,
        isAdult: Boolean
    ): Long {
        if (creatorId < 1) throw SodaException("올바른 크리에이터를 선택해 주세요.")

        val creator = memberRepository.findCreatorByIdOrNull(memberId = creatorId)
            ?: throw SodaException("올바른 크리에이터를 선택해 주세요.")

        val dateTimeFormatter = DateTimeFormatter.ofPattern("yyyy-MM-dd HH:mm")
        val startDate = LocalDateTime.parse(startDateString, dateTimeFormatter)
            .atZone(ZoneId.of("Asia/Seoul"))
            .withZoneSameInstant(ZoneId.of("UTC"))
            .toLocalDateTime()

        val nowDate = LocalDateTime.now()
            .atZone(ZoneId.of("Asia/Seoul"))
            .withZoneSameInstant(ZoneId.of("UTC"))
            .toLocalDateTime()

        if (startDate < nowDate) throw SodaException("노출 시작일은 현재시간 이후로 설정하셔야 합니다.")

        val endDate = LocalDateTime.parse(endDateString, dateTimeFormatter)
            .atZone(ZoneId.of("Asia/Seoul"))
            .withZoneSameInstant(ZoneId.of("UTC"))
            .toLocalDateTime()

        if (endDate < nowDate) throw SodaException("노출 종료일은 현재시간 이후로 설정하셔야 합니다.")
        if (endDate <= startDate) throw SodaException("노출 시작일은 노출 종료일 이전으로 설정하셔야 합니다.")

        val recommendCreatorBanner = RecommendLiveCreatorBanner(
            startDate = startDate,
            endDate = endDate,
            isAdult = isAdult
        )
        recommendCreatorBanner.creator = creator
        recommendCreatorBannerRepository.save(recommendCreatorBanner)

        val metadata = ObjectMetadata()
        metadata.contentLength = image.size
        val imagePath = s3Uploader.upload(
            inputStream = image.inputStream,
            bucket = bucket,
            filePath = "recommend_creator_banner/${recommendCreatorBanner.id}/${generateFileName()}",
            metadata = metadata
        )

        recommendCreatorBanner.image = imagePath

        return recommendCreatorBanner.id!!
    }

    @Transactional
    fun updateRecommendCreatorBanner(
        recommendCreatorBannerId: Long,
        image: MultipartFile?,
        creatorId: Long?,
        startDateString: String?,
        endDateString: String?,
        isAdult: Boolean?
    ) {
        val recommendCreatorBanner = recommendCreatorBannerRepository.findByIdOrNull(recommendCreatorBannerId)
            ?: throw SodaException("해당하는 추천라이브가 없습니다. 다시 확인해 주세요.")

        if (creatorId != null) {
            if (creatorId < 1) throw SodaException("올바른 크리에이터를 선택해 주세요.")

            val creator = memberRepository.findCreatorByIdOrNull(memberId = creatorId)
                ?: throw SodaException("올바른 크리에이터를 선택해 주세요.")

            recommendCreatorBanner.creator = creator
        }

        if (image != null) {
            val metadata = ObjectMetadata()
            metadata.contentLength = image.size
            val imagePath = s3Uploader.upload(
                inputStream = image.inputStream,
                bucket = bucket,
                filePath = "recommend_creator_banner/${recommendCreatorBanner.id}/${generateFileName()}",
                metadata = metadata
            )

            recommendCreatorBanner.image = imagePath
        }

        if (startDateString != null) {
            val dateTimeFormatter = DateTimeFormatter.ofPattern("yyyy-MM-dd HH:mm")
            val startDate = LocalDateTime.parse(startDateString, dateTimeFormatter)
                .atZone(ZoneId.of("Asia/Seoul"))
                .withZoneSameInstant(ZoneId.of("UTC"))
                .toLocalDateTime()

            val endDate = if (endDateString != null) {
                LocalDateTime.parse(endDateString, dateTimeFormatter)
                    .atZone(ZoneId.of("Asia/Seoul"))
                    .withZoneSameInstant(ZoneId.of("UTC"))
                    .toLocalDateTime()
            } else {
                null
            }

            if (endDate != null) {
                if (endDate <= startDate) {
                    throw SodaException("노출 시작일은 노출 종료일 이전으로 설정하셔야 합니다.")
                }

                recommendCreatorBanner.endDate = endDate
            } else {
                if (recommendCreatorBanner.endDate <= startDate) {
                    throw SodaException("노출 시작일은 노출 종료일 이전으로 설정하셔야 합니다.")
                }
            }

            recommendCreatorBanner.startDate = startDate
        } else if (endDateString != null) {
            val dateTimeFormatter = DateTimeFormatter.ofPattern("yyyy-MM-dd HH:mm")
            val endDate = LocalDateTime.parse(endDateString, dateTimeFormatter)
                .atZone(ZoneId.of("Asia/Seoul"))
                .withZoneSameInstant(ZoneId.of("UTC"))
                .toLocalDateTime()

            if (endDate <= recommendCreatorBanner.startDate) {
                throw SodaException("노출 종료일은 노출 시작일 이후로 설정하셔야 합니다.")
            }

            recommendCreatorBanner.endDate = endDate
        }

        if (isAdult != null) {
            recommendCreatorBanner.isAdult = isAdult
        }
    }

    @Transactional
    fun updateRecommendCreatorBannerOrders(firstOrders: Int, ids: List<Long>) {
        for (index in ids.indices) {
            val recommendCreatorBanner = recommendCreatorBannerRepository.findByIdOrNull(id = ids[index])

            if (recommendCreatorBanner != null) {
                recommendCreatorBanner.orders = firstOrders + index
            }
        }
    }

    @Transactional
    fun cancelNotStartedLive4hoursPassed() {
        val findRoomList = repository.getNotStartedLive4hoursPassed()

        for (room in findRoomList) {
            room.isActive = false

            val roomCancel = LiveRoomCancel("관리자에 의한 취소 - 노쇼")
            roomCancel.room = room
            roomCancelRepository.save(roomCancel)

            // 유료방인 경우 환불처리
            if (room.price > 0) {
                val bookerList = reservationRepository.getReservationBookerList(roomId = room.id!!)
                for (booker in bookerList) {
                    val useCan = canRepository.getCanUsedForLiveRoomNotRefund(
                        memberId = booker.id!!,
                        roomId = room.id!!,
                        canUsage = CanUsage.LIVE
                    ) ?: continue
                    useCan.isRefund = true

                    val useCanCalculate = useCanCalculateRepository.findByUseCanIdAndStatus(useCanId = useCan.id!!)
                    useCanCalculate.forEach {
                        it.status = UseCanCalculateStatus.REFUND

                        val charge = Charge(0, it.can, status = ChargeStatus.REFUND_CHARGE)
                        charge.title = "${it.can} 캔"
                        charge.useCan = useCan

                        when (it.paymentGateway) {
                            PaymentGateway.GOOGLE_IAP -> booker.googleRewardCan += charge.rewardCan
                            PaymentGateway.APPLE_IAP -> booker.appleRewardCan += charge.rewardCan
                            else -> booker.pgRewardCan += charge.rewardCan
                        }
                        charge.member = booker

                        val payment = Payment(
                            status = PaymentStatus.COMPLETE,
                            paymentGateway = it.paymentGateway
                        )
                        payment.method = "환불"
                        charge.payment = payment

                        chargeRepository.save(charge)
                    }
                }
            }

            // 예약현황 취소
            val pushTokenListMap = memberRepository.getPushTokenFromReservationList(room.id!!)
            reservationRepository.cancelReservation(roomId = room.id!!)

            // 라이브 취소 푸시 발송
            applicationEventPublisher.publishEvent(
                FcmEvent(
                    type = FcmEventType.CANCEL_LIVE,
                    title = room.member!!.nickname,
                    message = "라이브 취소 : ${room.title}",
                    recipientsMap = pushTokenListMap
                )
            )
        }
    }
}
