package kr.co.vividnext.sodalive.can.charge.event

import kr.co.vividnext.sodalive.can.charge.Charge
import kr.co.vividnext.sodalive.can.charge.ChargeRepository
import kr.co.vividnext.sodalive.can.charge.ChargeStatus
import kr.co.vividnext.sodalive.can.payment.Payment
import kr.co.vividnext.sodalive.can.payment.PaymentGateway
import kr.co.vividnext.sodalive.can.payment.PaymentStatus
import kr.co.vividnext.sodalive.common.SodaException
import kr.co.vividnext.sodalive.fcm.FcmEvent
import kr.co.vividnext.sodalive.fcm.FcmEventType
import kr.co.vividnext.sodalive.member.Member
import kr.co.vividnext.sodalive.member.MemberRepository
import kr.co.vividnext.sodalive.member.auth.AuthRepository
import org.springframework.context.ApplicationEventPublisher
import org.springframework.data.repository.findByIdOrNull
import org.springframework.stereotype.Service
import org.springframework.transaction.annotation.Transactional
import kotlin.math.ceil
import kotlin.math.round

@Service
class ChargeEventService(
    private val repository: ChargeEventRepository,
    private val authRepository: AuthRepository,
    private val memberRepository: MemberRepository,
    private val chargeRepository: ChargeRepository,
    private val chargeEventRepository: ChargeEventRepository,
    private val applicationEventPublisher: ApplicationEventPublisher
) {
    @Transactional
    fun applyChargeEvent(chargeId: Long, memberId: Long) {
        val charge = chargeRepository.findByIdOrNull(chargeId)
            ?: throw SodaException("이벤트가 적용되지 않았습니다.\n고객센터에 문의해 주세요.")

        val member = memberRepository.findByIdOrNull(memberId)
            ?: throw SodaException("이벤트가 적용되지 않았습니다.\n고객센터에 문의해 주세요.")

        if (member.auth != null) {
            val authDate = authRepository.getOldestCreatedAtByDi(member.auth!!.di)
            val memberIds = authRepository.getMemberIdsByDi(member.auth!!.di)

            var chargeCount = 0
            for (id in memberIds) {
                chargeCount += chargeRepository.getChargeCountAfterDate(memberId = id, authDate)
            }

            if (chargeCount > 1) {
                applyOtherEvent(charge, member)
            } else {
                applyFirstChargeEvent(charge, member)
            }
        } else {
            applyOtherEvent(charge, member)
        }
    }

    private fun applyOtherEvent(charge: Charge, member: Member) {
        val chargeEvent = repository.getChargeEvent()

        if (chargeEvent != null) {
            val eventChargeCount = chargeEventRepository.getPaymentCount(
                member = member,
                method = chargeEvent.title,
                startDate = chargeEvent.startDate,
                endDate = chargeEvent.endDate
            )

            if (eventChargeCount < chargeEvent.availableCount) {
                val additionalCan = round(charge.chargeCan * chargeEvent.addPercent).toInt()
                applyEvent(
                    additionalCan = additionalCan,
                    member = member,
                    paymentGateway = charge.payment?.paymentGateway!!,
                    method = chargeEvent.title
                )

                applicationEventPublisher.publishEvent(
                    FcmEvent(
                        type = FcmEventType.INDIVIDUAL,
                        title = chargeEvent.title,
                        message = "$additionalCan 캔이 추가 지급되었습니다.",
                        recipients = listOf(member.id!!),
                        isAuth = false
                    )
                )
            }
        }
    }

    private fun applyFirstChargeEvent(charge: Charge, member: Member) {
        val additionalCan = ceil(charge.chargeCan * 0.15).toInt()
        applyEvent(
            additionalCan = additionalCan,
            member = member,
            paymentGateway = charge.payment?.paymentGateway!!,
            method = "첫 충전 이벤트"
        )

        applicationEventPublisher.publishEvent(
            FcmEvent(
                type = FcmEventType.INDIVIDUAL,
                title = "첫 충전 이벤트",
                message = "$additionalCan 캔이 추가 지급되었습니다.",
                recipients = listOf(member.id!!),
                isAuth = false
            )
        )
    }

    private fun applyEvent(additionalCan: Int, member: Member, paymentGateway: PaymentGateway, method: String) {
        val eventCharge = Charge(0, additionalCan, status = ChargeStatus.EVENT)
        eventCharge.title = "$additionalCan 캔"
        eventCharge.member = member

        val payment = Payment(
            status = PaymentStatus.COMPLETE,
            paymentGateway = paymentGateway
        )
        payment.method = method
        eventCharge.payment = payment
        chargeRepository.save(eventCharge)

        when (paymentGateway) {
            PaymentGateway.GOOGLE_IAP -> member.charge(0, additionalCan, "aos")
            PaymentGateway.APPLE_IAP -> member.charge(0, additionalCan, "ios")
            else -> member.charge(0, additionalCan, "pg")
        }
    }
}
