package kr.co.vividnext.sodalive.content

import com.querydsl.core.types.dsl.Expressions
import com.querydsl.jpa.impl.JPAQueryFactory
import kr.co.vividnext.sodalive.content.QAudioContent.audioContent
import kr.co.vividnext.sodalive.content.QBundleAudioContent.bundleAudioContent
import kr.co.vividnext.sodalive.content.comment.QAudioContentComment.audioContentComment
import kr.co.vividnext.sodalive.content.like.QAudioContentLike.audioContentLike
import kr.co.vividnext.sodalive.content.main.GetAudioContentMainItem
import kr.co.vividnext.sodalive.content.main.GetAudioContentRankingItem
import kr.co.vividnext.sodalive.content.main.GetNewContentUploadCreator
import kr.co.vividnext.sodalive.content.main.QGetAudioContentMainItem
import kr.co.vividnext.sodalive.content.main.QGetAudioContentRankingItem
import kr.co.vividnext.sodalive.content.main.banner.AudioContentBanner
import kr.co.vividnext.sodalive.content.main.banner.QAudioContentBanner.audioContentBanner
import kr.co.vividnext.sodalive.content.main.curation.AudioContentCuration
import kr.co.vividnext.sodalive.content.main.curation.QAudioContentCuration.audioContentCuration
import kr.co.vividnext.sodalive.content.order.QOrder.order
import kr.co.vividnext.sodalive.content.theme.QAudioContentTheme.audioContentTheme
import kr.co.vividnext.sodalive.event.QEvent.event
import kr.co.vividnext.sodalive.member.MemberRole
import kr.co.vividnext.sodalive.member.QMember.member
import org.springframework.data.jpa.repository.JpaRepository
import org.springframework.stereotype.Repository
import java.time.LocalDateTime

@Repository
interface AudioContentRepository : JpaRepository<AudioContent, Long>, AudioContentQueryRepository

interface AudioContentQueryRepository {
    fun findByIdAndActive(contentId: Long): AudioContent?
    fun findByIdAndCreatorId(contentId: Long, creatorId: Long): AudioContent?
    fun findBundleByContentId(contentId: Long): List<AudioContent>
    fun findByCreatorId(
        creatorId: Long,
        isAdult: Boolean = false,
        sortType: SortType = SortType.NEWEST,
        offset: Long = 0,
        limit: Long = 10
    ): List<AudioContent>

    fun findTotalCountByCreatorId(creatorId: Long, isAdult: Boolean = false): Int
    fun getCreatorOtherContentList(
        cloudfrontHost: String,
        contentId: Long,
        creatorId: Long,
        isAdult: Boolean
    ): List<OtherContentResponse>

    fun getSameThemeOtherContentList(
        cloudfrontHost: String,
        contentId: Long,
        themeId: Long,
        isAdult: Boolean
    ): List<OtherContentResponse>

    fun findByTheme(
        cloudfrontHost: String,
        theme: String = "",
        isAdult: Boolean = false,
        offset: Long = 0,
        limit: Long = 20
    ): List<GetAudioContentMainItem>

    fun findByThemeFor2Weeks(
        cloudfrontHost: String,
        theme: String = "",
        isAdult: Boolean = false,
        offset: Long = 0,
        limit: Long = 20
    ): List<GetAudioContentMainItem>

    fun totalCountNewContentFor2Weeks(theme: String, isAdult: Boolean): Int

    fun getNewContentUploadCreatorList(
        cloudfrontHost: String,
        isAdult: Boolean = false
    ): List<GetNewContentUploadCreator>

    fun getAudioContentMainBannerList(isAdult: Boolean): List<AudioContentBanner>
    fun getAudioContentCurations(isAdult: Boolean): List<AudioContentCuration>
    fun findAudioContentByCurationId(
        curationId: Long,
        cloudfrontHost: String,
        isAdult: Boolean
    ): List<GetAudioContentMainItem>

    fun getAudioContentRanking(
        cloudfrontHost: String,
        isAdult: Boolean,
        startDate: LocalDateTime,
        endDate: LocalDateTime,
        offset: Long = 0,
        limit: Long = 12,
        sortType: String = "매출"
    ): List<GetAudioContentRankingItem>
}

@Repository
class AudioContentQueryRepositoryImpl(private val queryFactory: JPAQueryFactory) : AudioContentQueryRepository {
    override fun findByIdAndActive(contentId: Long): AudioContent? {
        return queryFactory
            .selectFrom(audioContent)
            .where(
                audioContent.isActive.isTrue
                    .and(audioContent.id.eq(contentId))
            )
            .fetchOne()
    }

    override fun findByIdAndCreatorId(contentId: Long, creatorId: Long): AudioContent? {
        return queryFactory
            .selectFrom(audioContent)
            .where(
                audioContent.isActive.isTrue
                    .and(audioContent.id.eq(contentId))
                    .and(audioContent.member.id.eq(creatorId))
            )
            .fetchOne()
    }

    // 해당 컨텐츠가 속한 묶음(번들) 상품 리스트 검색
    override fun findBundleByContentId(contentId: Long): List<AudioContent> {
        return queryFactory
            .select(bundleAudioContent.parent)
            .from(bundleAudioContent)
            .where(
                bundleAudioContent.child.id.eq(contentId)
                    .and(bundleAudioContent.child.isActive.isTrue)
            )
            .fetch()
    }

    override fun findByCreatorId(
        creatorId: Long,
        isAdult: Boolean,
        sortType: SortType,
        offset: Long,
        limit: Long
    ): List<AudioContent> {
        val orderBy = when (sortType) {
            SortType.NEWEST -> audioContent.createdAt.desc()
            SortType.PRICE_HIGH -> audioContent.price.desc()
            SortType.PRICE_LOW -> audioContent.price.asc()
        }

        var where = audioContent.isActive.isTrue
            .and(audioContent.member.id.eq(creatorId))

        if (!isAdult) {
            where = where.and(audioContent.isAdult.isFalse)
        }

        return queryFactory
            .selectFrom(audioContent)
            .where(where)
            .offset(offset)
            .limit(limit)
            .orderBy(orderBy)
            .fetch()
    }

    override fun findTotalCountByCreatorId(
        creatorId: Long,
        isAdult: Boolean
    ): Int {
        var where = audioContent.isActive.isTrue
            .and(audioContent.member.id.eq(creatorId))

        if (!isAdult) {
            where = where.and(audioContent.isAdult.isFalse)
        }

        return queryFactory
            .selectFrom(audioContent)
            .where(where)
            .fetch()
            .size
    }

    override fun getCreatorOtherContentList(
        cloudfrontHost: String,
        contentId: Long,
        creatorId: Long,
        isAdult: Boolean
    ): List<OtherContentResponse> {
        var where = audioContent.id.ne(contentId)
            .and(audioContent.member.id.eq(creatorId))
            .and(audioContent.isActive.isTrue)

        if (!isAdult) {
            where = where.and(audioContent.isAdult.isFalse)
        }

        return queryFactory
            .select(
                QOtherContentResponse(
                    audioContent.id,
                    audioContent.title,
                    audioContent.coverImage.prepend("$cloudfrontHost/")
                )
            )
            .from(audioContent)
            .where(where)
            .offset(0)
            .limit(10)
            .orderBy(Expressions.numberTemplate(Double::class.java, "function('rand')").asc())
            .fetch()
    }

    override fun getSameThemeOtherContentList(
        cloudfrontHost: String,
        contentId: Long,
        themeId: Long,
        isAdult: Boolean
    ): List<OtherContentResponse> {
        var where = audioContent.id.ne(contentId)
            .and(audioContent.theme.id.eq(themeId))
            .and(audioContent.isActive.isTrue)

        if (!isAdult) {
            where = where.and(audioContent.isAdult.isFalse)
        }

        return queryFactory
            .select(
                QOtherContentResponse(
                    audioContent.id,
                    audioContent.title,
                    audioContent.coverImage.prepend("$cloudfrontHost/")
                )
            )
            .from(audioContent)
            .where(where)
            .offset(0)
            .limit(10)
            .orderBy(Expressions.numberTemplate(Double::class.java, "function('rand')").asc())
            .fetch()
    }

    override fun findByTheme(
        cloudfrontHost: String,
        theme: String,
        isAdult: Boolean,
        offset: Long,
        limit: Long
    ): List<GetAudioContentMainItem> {
        var where = audioContent.isActive.isTrue

        if (!isAdult) {
            where = where.and(audioContent.isAdult.isFalse)
        }

        if (theme.isNotBlank()) {
            where = where.and(audioContentTheme.theme.eq(theme))
        }

        return queryFactory
            .select(
                QGetAudioContentMainItem(
                    audioContent.id,
                    audioContent.coverImage.prepend("/").prepend(cloudfrontHost),
                    audioContent.title,
                    member.id,
                    member.profileImage.prepend("/").prepend(cloudfrontHost),
                    member.nickname
                )
            )
            .from(audioContent)
            .innerJoin(audioContent.member, member)
            .innerJoin(audioContent.theme, audioContentTheme)
            .where(where)
            .offset(offset)
            .limit(limit)
            .orderBy(audioContent.createdAt.desc())
            .fetch()
    }

    override fun totalCountNewContentFor2Weeks(theme: String, isAdult: Boolean): Int {
        var where = audioContent.isActive.isTrue
            .and(audioContent.createdAt.goe(LocalDateTime.now().minusWeeks(2)))

        if (!isAdult) {
            where = where.and(audioContent.isAdult.isFalse)
        }

        if (theme.isNotBlank()) {
            where = where.and(audioContentTheme.theme.eq(theme))
        }

        return queryFactory
            .select(audioContent.id)
            .from(audioContent)
            .innerJoin(audioContent.member, member)
            .innerJoin(audioContent.theme, audioContentTheme)
            .where(where)
            .fetch()
            .size
    }

    override fun findByThemeFor2Weeks(
        cloudfrontHost: String,
        theme: String,
        isAdult: Boolean,
        offset: Long,
        limit: Long
    ): List<GetAudioContentMainItem> {
        var where = audioContent.isActive.isTrue
            .and(audioContent.createdAt.goe(LocalDateTime.now().minusWeeks(2)))

        if (!isAdult) {
            where = where.and(audioContent.isAdult.isFalse)
        }

        if (theme.isNotBlank()) {
            where = where.and(audioContentTheme.theme.eq(theme))
        }

        return queryFactory
            .select(
                QGetAudioContentMainItem(
                    audioContent.id,
                    audioContent.coverImage.prepend("/").prepend(cloudfrontHost),
                    audioContent.title,
                    member.id,
                    member.profileImage.prepend("/").prepend(cloudfrontHost),
                    member.nickname
                )
            )
            .from(audioContent)
            .innerJoin(audioContent.member, member)
            .innerJoin(audioContent.theme, audioContentTheme)
            .where(where)
            .offset(offset)
            .limit(limit)
            .orderBy(audioContent.createdAt.desc())
            .fetch()
    }

    override fun getNewContentUploadCreatorList(
        cloudfrontHost: String,
        isAdult: Boolean
    ): List<GetNewContentUploadCreator> {
        var where = audioContent.createdAt.after(LocalDateTime.now().minusWeeks(2))
            .and(audioContent.isActive.isTrue)

        if (!isAdult) {
            where = where.and(audioContent.isAdult.isFalse)
        }

        return queryFactory
            .select(member)
            .from(audioContent)
            .innerJoin(audioContent.member, member)
            .where(where)
            .groupBy(member.id)
            .orderBy(Expressions.numberTemplate(Double::class.java, "function('rand')").asc())
            .limit(20)
            .fetch()
            .asSequence()
            .map {
                GetNewContentUploadCreator(
                    it.id!!,
                    it.nickname,
                    creatorProfileImageUrl = if (it.profileImage != null) {
                        "$cloudfrontHost/${it.profileImage}"
                    } else {
                        "$cloudfrontHost/profile/default-profile.png"
                    }
                )
            }
            .toList()
    }

    override fun getAudioContentMainBannerList(isAdult: Boolean): List<AudioContentBanner> {
        var where = audioContentBanner.isActive.isTrue

        if (!isAdult) {
            where = where.and(audioContentBanner.isAdult.isFalse)
        }

        return queryFactory
            .selectFrom(audioContentBanner)
            .leftJoin(audioContentBanner.event, event)
            .leftJoin(audioContentBanner.creator, member)
            .where(where)
            .orderBy(audioContentBanner.orders.asc())
            .fetch()
    }

    override fun getAudioContentCurations(isAdult: Boolean): List<AudioContentCuration> {
        var where = audioContentCuration.isActive.isTrue

        if (!isAdult) {
            where = where.and(audioContentCuration.isAdult.isFalse)
        }

        return queryFactory
            .selectFrom(audioContentCuration)
            .where(where)
            .orderBy(audioContentCuration.orders.asc())
            .fetch()
    }

    override fun findAudioContentByCurationId(
        curationId: Long,
        cloudfrontHost: String,
        isAdult: Boolean
    ): List<GetAudioContentMainItem> {
        var where = audioContent.isActive.isTrue
            .and(audioContent.member.isNotNull)
            .and(audioContent.duration.isNotNull)
            .and(audioContent.member.isActive.isTrue)
            .and(audioContent.curation.id.eq(curationId))

        if (!isAdult) {
            where = where.and(audioContent.isAdult.isFalse)
        }

        return queryFactory
            .select(
                QGetAudioContentMainItem(
                    audioContent.id,
                    audioContent.coverImage.prepend("/").prepend(cloudfrontHost),
                    audioContent.title,
                    member.id,
                    member.profileImage
                        .prepend("/")
                        .prepend(cloudfrontHost),
                    member.nickname
                )
            )
            .from(audioContent)
            .innerJoin(audioContent.member, member)
            .leftJoin(audioContent.curation, audioContentCuration)
            .where(where)
            .orderBy(audioContent.id.desc())
            .fetch()
    }

    override fun getAudioContentRanking(
        cloudfrontHost: String,
        isAdult: Boolean,
        startDate: LocalDateTime,
        endDate: LocalDateTime,
        offset: Long,
        limit: Long,
        sortType: String
    ): List<GetAudioContentRankingItem> {
        var where = audioContent.isActive.isTrue
            .and(audioContent.member.isActive.isTrue)
            .and(audioContent.member.isNotNull)
            .and(audioContent.member.role.eq(MemberRole.CREATOR))
            .and(audioContent.duration.isNotNull)
            .and(audioContentTheme.isActive.isTrue)

        if (!isAdult) {
            where = where.and(audioContent.isAdult.isFalse)
        }

        var select = queryFactory
            .select(
                QGetAudioContentRankingItem(
                    audioContent.id,
                    audioContent.title,
                    audioContent.coverImage.prepend("/").prepend(cloudfrontHost),
                    audioContentTheme.theme,
                    audioContent.price,
                    audioContent.duration,
                    member.id,
                    member.nickname
                )
            )

        select = when (sortType) {
            "후원" -> {
                select
                    .from(audioContentComment)
                    .innerJoin(audioContentComment.audioContent, audioContent)
                    .innerJoin(audioContentComment.audioContent.member, member)
                    .innerJoin(audioContentComment.audioContent.theme, audioContentTheme)
                    .where(
                        where
                            .and(audioContentComment.isActive.isTrue)
                            .and(audioContentComment.donationCan.gt(0))
                            .and(audioContentComment.createdAt.goe(startDate))
                            .and(audioContentComment.createdAt.lt(endDate))
                    )
                    .groupBy(audioContent.id)
                    .orderBy(audioContentComment.donationCan.sum().desc(), audioContent.createdAt.asc())
            }

            "댓글" -> {
                select
                    .from(audioContentComment)
                    .innerJoin(audioContentComment.audioContent, audioContent)
                    .innerJoin(audioContentComment.audioContent.member, member)
                    .innerJoin(audioContentComment.audioContent.theme, audioContentTheme)
                    .where(
                        where
                            .and(audioContentComment.isActive.isTrue)
                            .and(audioContentComment.createdAt.goe(startDate))
                            .and(audioContentComment.createdAt.lt(endDate))
                    )
                    .groupBy(audioContentComment.audioContent.id)
                    .orderBy(audioContentComment.id.count().desc(), audioContent.createdAt.asc())
            }

            "좋아요" -> {
                select
                    .from(audioContentLike)
                    .innerJoin(audioContentLike.audioContent, audioContent)
                    .innerJoin(audioContentLike.audioContent.member, member)
                    .innerJoin(audioContentLike.audioContent.theme, audioContentTheme)
                    .where(
                        where
                            .and(audioContentLike.isActive.isTrue)
                            .and(audioContentLike.createdAt.goe(startDate))
                            .and(audioContentLike.createdAt.lt(endDate))
                    )
                    .groupBy(audioContentLike.audioContent.id)
                    .orderBy(audioContentLike.id.count().desc(), audioContent.createdAt.asc())
            }

            else -> {
                select
                    .from(order)
                    .innerJoin(order.audioContent, audioContent)
                    .innerJoin(audioContent.member, member)
                    .innerJoin(audioContent.theme, audioContentTheme)
                    .where(
                        where
                            .and(order.isActive.isTrue)
                            .and(order.createdAt.goe(startDate))
                            .and(order.createdAt.lt(endDate))
                    )
                    .groupBy(audioContent.id)
                    .orderBy(order.can.sum().desc(), audioContent.createdAt.asc())
            }
        }

        return select
            .offset(offset)
            .limit(limit)
            .fetch()
    }
}
