package kr.co.vividnext.sodalive.member

import com.querydsl.jpa.impl.JPAQueryFactory
import kr.co.vividnext.sodalive.fcm.GetMessageRecipientPushTokenResponse
import kr.co.vividnext.sodalive.fcm.QGetMessageRecipientPushTokenResponse
import kr.co.vividnext.sodalive.live.reservation.QLiveReservation.liveReservation
import kr.co.vividnext.sodalive.live.room.QLiveRoom.liveRoom
import kr.co.vividnext.sodalive.member.QMember.member
import kr.co.vividnext.sodalive.member.auth.QAuth.auth
import kr.co.vividnext.sodalive.member.block.BlockMemberRepository
import kr.co.vividnext.sodalive.member.following.QCreatorFollowing.creatorFollowing
import kr.co.vividnext.sodalive.member.nickname.QNicknameChangeLog.nicknameChangeLog
import kr.co.vividnext.sodalive.member.notification.QMemberNotification.memberNotification
import kr.co.vividnext.sodalive.message.QMessage.message
import org.springframework.data.jpa.repository.JpaRepository
import org.springframework.stereotype.Repository

@Repository
interface MemberRepository : JpaRepository<Member, Long>, MemberQueryRepository {
    fun findByEmail(email: String): Member?
    fun findByNickname(nickname: String): Member?
}

interface MemberQueryRepository {
    fun findByPushToken(pushToken: String): List<Member>
    fun findByNicknameAndOtherCondition(nickname: String, memberId: Long): List<Member>
    fun findCreatorByIdOrNull(memberId: Long): Member?
    fun getAllRecipientPushTokens(isAuth: Boolean, container: String): List<List<String>>
    fun getCreateLiveRoomNotificationRecipientPushTokens(
        creatorId: Long,
        isAuth: Boolean,
        container: String
    ): List<List<String>>

    fun getStartLiveRoomNotificationRecipientPushTokens(
        creatorId: Long,
        roomId: Long,
        isAuth: Boolean,
        container: String
    ): List<List<String>>

    fun getUploadContentNotificationRecipientPushTokens(
        creatorId: Long,
        isAuth: Boolean,
        container: String
    ): List<List<String>>

    fun getMessageRecipientPushToken(messageId: Long): GetMessageRecipientPushTokenResponse
    fun getIndividualRecipientPushTokens(recipients: List<Long>, isAuth: Boolean): Map<String, List<List<String>>>
    fun getChangeNicknamePrice(memberId: Long): GetChangeNicknamePriceResponse
    fun getMemberByEmail(email: String): Member?

    fun getChangeNoticeRecipientPushTokens(creatorId: Long): Map<String, List<List<String>>>
    fun getPushTokenFromReservationList(roomId: Long): Map<String, List<List<String>>>
}

@Repository
class MemberQueryRepositoryImpl(
    private val queryFactory: JPAQueryFactory,
    private val blockMemberRepository: BlockMemberRepository
) : MemberQueryRepository {
    override fun findByPushToken(pushToken: String): List<Member> {
        return queryFactory
            .selectFrom(member)
            .where(member.pushToken.eq(pushToken))
            .fetch()
    }

    override fun findByNicknameAndOtherCondition(nickname: String, memberId: Long): List<Member> {
        return queryFactory
            .selectFrom(member)
            .where(
                member.nickname.containsIgnoreCase(nickname)
                    .and(member.id.ne(memberId))
                    .and(member.role.ne(MemberRole.ADMIN))
                    .and(member.role.ne(MemberRole.AGENT))
            )
            .fetch()
    }

    override fun findCreatorByIdOrNull(memberId: Long): Member? {
        return queryFactory
            .selectFrom(member)
            .where(
                member.id.eq(memberId)
                    .and(member.role.eq(MemberRole.CREATOR))
            )
            .fetchFirst()
    }

    override fun getAllRecipientPushTokens(isAuth: Boolean, container: String): List<List<String>> {
        var where = member.isActive.isTrue
            .and(member.email.notIn("admin@sodalive.net"))
            .and(member.container.eq(container))
            .and(member.pushToken.isNotNull)

        if (isAuth) {
            where = where.and(member.auth.isNotNull)
        }

        return queryFactory
            .select(member.pushToken)
            .from(member)
            .leftJoin(member.auth, auth)
            .where(where)
            .fetch()
            .toSet()
            .chunked(500)
    }

    override fun getCreateLiveRoomNotificationRecipientPushTokens(
        creatorId: Long,
        isAuth: Boolean,
        container: String
    ): List<List<String>> {
        val member = QMember.member
        val creator = QMember.member

        var where = creatorFollowing.isActive.isTrue
            .and(creatorFollowing.creator.id.eq(creatorId))
            .and(creatorFollowing.member.notification.live.isTrue)
            .and(creatorFollowing.member.container.eq(container))
            .and(creatorFollowing.member.email.notIn("admin@sodalive.net"))
            .and(
                creatorFollowing.member.id.notIn(
                    blockMemberRepository.getBlockedMemberList(creatorId)
                )
            )
            .and(creatorFollowing.member.pushToken.isNotNull)
            .or(member.id.eq(4).and(member.pushToken.isNotNull))

        if (isAuth) {
            where = where.and(member.auth.isNotNull)
        }

        return queryFactory
            .select(creatorFollowing.member.pushToken)
            .from(creatorFollowing)
            .innerJoin(creatorFollowing.creator, creator)
            .innerJoin(creatorFollowing.member, member)
            .innerJoin(member.notification, memberNotification)
            .leftJoin(member.auth, auth)
            .where(where)
            .fetch()
            .toSet()
            .chunked(500)
    }

    override fun getStartLiveRoomNotificationRecipientPushTokens(
        creatorId: Long,
        roomId: Long,
        isAuth: Boolean,
        container: String
    ): List<List<String>> {
        val member = QMember.member
        val creator = QMember.member

        var where = creatorFollowing.isActive.isTrue
            .and(creatorFollowing.creator.id.eq(creatorId))
            .and(creatorFollowing.member.notification.live.isTrue)
            .and(creatorFollowing.member.container.eq(container))
            .and(creatorFollowing.member.email.notIn("admin@sodalive.net"))
            .and(
                creatorFollowing.member.id.notIn(
                    blockMemberRepository.getBlockedMemberList(creatorId)
                )
            )
            .and(creatorFollowing.member.pushToken.isNotNull)
            .or(creatorFollowing.member.id.eq(4).and(member.pushToken.isNotNull))

        if (isAuth) {
            where = where.and(creatorFollowing.member.auth.isNotNull)
        }

        val followingMemberPushToken = queryFactory
            .select(creatorFollowing.member.pushToken)
            .from(creatorFollowing)
            .innerJoin(creatorFollowing.creator, creator)
            .innerJoin(creatorFollowing.member, member)
            .innerJoin(member.notification, memberNotification)
            .leftJoin(member.auth, auth)
            .where(where)
            .fetch()

        where = liveReservation.isActive.isTrue
            .and(liveReservation.member.notification.live.isTrue)
            .and(liveReservation.member.container.eq(container))
            .and(liveReservation.member.email.notIn("admin@sodalive.net"))
            .and(liveReservation.member.pushToken.isNotNull)
            .and(liveReservation.room.id.eq(roomId))
            .and(
                liveReservation.member.id.notIn(
                    blockMemberRepository.getBlockedMemberList(creatorId)
                )
            )

        if (isAuth) {
            where = where.and(liveReservation.member.auth.isNotNull)
        }

        val reservationMemberPushToken = queryFactory
            .select(liveReservation.member.pushToken)
            .from(liveReservation)
            .innerJoin(liveReservation.member, member)
            .innerJoin(member.notification, memberNotification)
            .leftJoin(member.auth, auth)
            .where(where)
            .fetch()

        return (followingMemberPushToken + reservationMemberPushToken)
            .toSet()
            .chunked(500)
    }

    override fun getUploadContentNotificationRecipientPushTokens(
        creatorId: Long,
        isAuth: Boolean,
        container: String
    ): List<List<String>> {
        val member = QMember.member
        val creator = QMember.member

        var where = creatorFollowing.isActive.isTrue
            .and(creatorFollowing.creator.id.eq(creatorId))
            .and(creatorFollowing.member.email.notIn("admin@sodalive.net"))
            .and(creatorFollowing.member.container.eq(container))
            .and(creatorFollowing.member.notification.uploadContent.isTrue)
            .and(
                creatorFollowing.member.id.notIn(
                    blockMemberRepository.getBlockedMemberList(creatorId)
                )
            )
            .and(creatorFollowing.member.pushToken.isNotNull)
            .or(member.id.eq(4).and(member.pushToken.isNotNull))

        if (isAuth) {
            where = where.and(member.auth.isNotNull)
        }

        return queryFactory
            .select(creatorFollowing.member.pushToken)
            .from(creatorFollowing)
            .innerJoin(creatorFollowing.creator, creator)
            .innerJoin(creatorFollowing.member, member)
            .innerJoin(member.notification, memberNotification)
            .leftJoin(member.auth, auth)
            .where(where)
            .fetch()
            .toSet()
            .chunked(500)
    }

    override fun getMessageRecipientPushToken(messageId: Long): GetMessageRecipientPushTokenResponse {
        return queryFactory
            .select(
                QGetMessageRecipientPushTokenResponse(
                    member.pushToken,
                    member.container
                )
            )
            .from(message)
            .innerJoin(message.recipient, member)
            .innerJoin(member.notification, memberNotification)
            .where(
                message.id.eq(messageId)
                    .and(member.pushToken.isNotNull)
                    .and(memberNotification.message.isTrue)
            )
            .fetchFirst()
    }

    override fun getIndividualRecipientPushTokens(
        recipients: List<Long>,
        isAuth: Boolean
    ): Map<String, List<List<String>>> {
        var where = member.isActive.isTrue
            .and(member.email.notIn("admin@sodalive.net"))
            .and(member.id.`in`(*recipients.toTypedArray()))
            .and(member.pushToken.isNotNull)

        if (isAuth) {
            where = where.and(member.auth.isNotNull)
        }

        val aosPushTokens = queryFactory
            .select(member.pushToken)
            .from(member)
            .leftJoin(member.auth, auth)
            .where(where.and(member.container.eq("aos")))
            .fetch()
            .toSet()
            .chunked(500)

        val iosPushTokens = queryFactory
            .select(member.pushToken)
            .from(member)
            .leftJoin(member.auth, auth)
            .where(where.and(member.container.eq("ios")))
            .fetch()
            .toSet()
            .chunked(500)

        return mapOf("aos" to aosPushTokens, "ios" to iosPushTokens)
    }

    override fun getChangeNicknamePrice(memberId: Long): GetChangeNicknamePriceResponse {
        val changeCount = queryFactory
            .select(nicknameChangeLog.id)
            .from(nicknameChangeLog)
            .where(nicknameChangeLog.member.id.eq(memberId))
            .fetch()
            .count()

        return GetChangeNicknamePriceResponse(
            price = if (changeCount > 0) {
                100
            } else {
                0
            }
        )
    }

    override fun getMemberByEmail(email: String): Member? {
        return queryFactory
            .selectFrom(member)
            .where(member.email.eq(email))
            .fetchOne()
    }

    override fun getChangeNoticeRecipientPushTokens(creatorId: Long): Map<String, List<List<String>>> {
        val member = QMember.member
        val creator = QMember.member

        val where = creatorFollowing.isActive.isTrue
            .and(creatorFollowing.creator.id.eq(creatorId))
            .and(creatorFollowing.member.email.notIn("admin@sodalive.net"))
            .and(
                creatorFollowing.member.id.notIn(
                    blockMemberRepository.getBlockedMemberList(creatorId)
                )
            )
            .and(creatorFollowing.member.pushToken.isNotNull)

        val aosPushTokens = queryFactory
            .select(creatorFollowing.member.pushToken)
            .from(creatorFollowing)
            .innerJoin(creatorFollowing.creator, creator)
            .innerJoin(creatorFollowing.member, member)
            .where(where.and(member.container.eq("aos")))
            .fetch()
            .toSet()
            .chunked(500)

        val iosPushTokens = queryFactory
            .select(creatorFollowing.member.pushToken)
            .from(creatorFollowing)
            .innerJoin(creatorFollowing.creator, creator)
            .innerJoin(creatorFollowing.member, member)
            .where(where.and(member.container.eq("ios")))
            .fetch()
            .toSet()
            .chunked(500)

        return mapOf("aos" to aosPushTokens, "ios" to iosPushTokens)
    }

    override fun getPushTokenFromReservationList(roomId: Long): Map<String, List<List<String>>> {
        val where = liveRoom.id.eq(roomId)
            .and(liveReservation.isActive.isTrue)

        val aosPushTokens = queryFactory
            .select(liveReservation.member.pushToken)
            .from(liveReservation)
            .innerJoin(liveReservation.room, liveRoom)
            .innerJoin(liveReservation.member, member)
            .where(where.and(member.container.eq("aos")))
            .fetch()
            .toSet()
            .chunked(500)

        val iosPushTokens = queryFactory
            .select(liveReservation.member.pushToken)
            .from(liveReservation)
            .innerJoin(liveReservation.room, liveRoom)
            .innerJoin(liveReservation.member, member)
            .where(where.and(member.container.eq("ios")))
            .fetch()
            .toSet()
            .chunked(500)

        return mapOf("aos" to aosPushTokens, "ios" to iosPushTokens)
    }
}
